import {AdminPages, TopBarButtons} from "@/js/admin/admin_pages";
import {Api} from "@/js/admin/Api";
import {
    CustomerModel,
    Ingredient,
    IngredientsModel,
    Order,
    OrderModel,
    Products,
    ProductsModel, Stock, StockModel
} from "@/js/declarations";
import {ApiHost} from "@/js/vars";
import {MDCDialog} from "@material/dialog";
import {MDCRipple} from "@material/ripple";
import {MDCTextField} from "@material/textfield";
import {MDCSelect} from '@material/select';

export class PageStock {
    private _appContainer: HTMLElement;
    private _context: AdminPages;
    private _api: Api;
    private _pageContainer?: HTMLElement;
    private _dialog?: MDCDialog;
    private _dialogStruct?: DialogStruct;
    private _pendingAction: boolean;

    constructor(appContainer: HTMLElement, adminPages: AdminPages) {
        this._appContainer = appContainer;
        this._api = new Api();
        this._context = adminPages;
        this._pendingAction = false;
        this._context.setTitle("Estoque");
        this.buildPage();
    }

    public destroy(): void {
        this._dialog?.root.remove();
    }

    private async buildPage() {
        const htmlData = await fetch("/template_estoque");
        const htmlDataBody = await htmlData.text();
        this._appContainer.innerHTML = htmlDataBody;

        const dialogHtmlData = await fetch("/template_estoque_dialog");
        const dialogHtmlDataBody = await dialogHtmlData.text();

        const dialogEl = document.createElement('div');
        dialogEl.className = "mdc-dialog mdc-dialog--fullscreen";
        dialogEl.style.zIndex = "1000";
        dialogEl.innerHTML = dialogHtmlDataBody;
        dialogEl.id = "dialog-estoque";

        const docBody = document.querySelector('body');

        if (docBody) {
            docBody.insertAdjacentElement('afterbegin', dialogEl);
        }

        this._dialog = new MDCDialog(document.getElementById('dialog-estoque') || new HTMLElement());

        this._dialog.listen('MDCDialog:closing', () => {
            if (!this._pendingAction) {
                this.destroy();
                this.buildPage();
            }
        });

        this._dialogStruct = {
            header: document.getElementById("txt-head-dialog") || new HTMLElement(),
            body: document.getElementById("estoque-dialog-body") || new HTMLElement(),
            buttonOk: document.getElementById("button-dialog-ok") || new HTMLElement(),
            buttonOkLabel: document.getElementById("btn-label-dialog") || new HTMLElement(),
        };

        this._pageContainer = document.querySelector('.container-gerenciador-estoque') || new HTMLElement();

        await this.getProdutos();
    }

    private async getProdutos() {
        const stocks: StockModel = await this._api.getFromApi("/estoque/listar") as StockModel;
        if (stocks.estoques.length === 0) {
            if (this._pageContainer) {
                const table = this._pageContainer.querySelector('#table-root') as HTMLElement;
                table.remove();
                this._pageContainer.innerHTML += `
                <div class="mdc-typography--subtitle1 txt-empty-max-width no-ingredient">Não há produtos/estoque cadastrados</div>
                `
            }
        } else {
            for (const stock of stocks.estoques) {
                if (this._pageContainer) {
                    const product: ProductsModel = await this._api.getFromApi("/produto", stock.idProduto) as ProductsModel;
                    const table = this._pageContainer.querySelector('#table-estoque') as HTMLElement;
                    table.innerHTML += `
                <tr class="mdc-data-table__row row-estoque" id="${stock.idProduto}">
                  <th class="mdc-data-table__cell" scope="row">${product.produtos[0].nomeGeladinho}</th>
                  <td class="mdc-data-table__cell">${stock.quantidadeEstoque}</td>
                </tr>
                `
                }
            }
            const rows = document.getElementsByClassName('row-estoque');
            if (rows) {
                for (let i = 0; i < rows.length; i++) {
                    const row = rows[i];
                    row.addEventListener('click', (event) => this.editStock(event, row.id));
                }
            }
        }
    }

    private editStock = async (event: Event, stockId: string): Promise<void> => {
        if (this._dialog && this._dialogStruct) {
            this._dialogStruct.body.innerHTML = "";
            this._dialog.open();
            this._dialogStruct.header.innerText = "Editar Estoque";
            this._dialogStruct.buttonOkLabel.innerText = "Salvar";
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-stock-dialog-name", "Produto", TextFieldTypes.SINGLE, "text"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-stock-dialog-qty", "Quantidade", TextFieldTypes.SINGLE, "number"));

            const stocks: StockModel = await this._api.getFromApi("/estoque/get", stockId) as StockModel;
            const stock: Stock = stocks.estoques[0];
            const product: ProductsModel = await this._api.getFromApi("/produto", stock.idProduto) as ProductsModel;

            const listCustomerCt = document.createElement('div');
            listCustomerCt.style.height = "fit-content";

            const dialogEls: DialogEls = {
                productName: new MDCTextField(document.getElementById("txt-stock-dialog-name") || new HTMLElement()),
                quantity: new MDCTextField(document.getElementById("txt-stock-dialog-qty") || new HTMLElement()),
            }

            dialogEls.quantity.getDefaultFoundation().setValue(stock.quantidadeEstoque.toString());
            dialogEls.productName.getDefaultFoundation().setValue(product.produtos[0].nomeGeladinho);
            dialogEls.productName.getDefaultFoundation().setDisabled(true);

            this._dialogStruct.buttonOk.addEventListener('click', async () => {
                if (this.checkData(dialogEls)) {
                    this._pendingAction = true;
                    if (this._dialogStruct) {
                        this._dialogStruct.buttonOk.setAttribute("disabled", "");
                    }
                    if (stockId) {
                        const stockData: { quantidadeEstoque: number } = {
                            quantidadeEstoque: Number(dialogEls.quantity.getDefaultFoundation().getValue())
                        }

                        const res = await this._api.putToApi("/estoque/editar", stockData, stockId);
                        this._dialog?.close();
                        if (res["error"]) {
                            this._context.showSnackbar("Erro ao Editar Estoque");
                            if (this._dialogStruct) {
                                this._dialogStruct.buttonOk.removeAttribute("disabled");
                            }
                        } else {
                            this._context.showSnackbar("Estoque Editado com Sucesso");
                            this.destroy();
                            this.buildPage();
                        }
                    }
                } else {
                    this._context.showSnackbar("Preencha os Campos Obrigatórios");
                }
            })

        }
    }

    private newTextField(id: string, label: string, type: TextFieldTypes, typeInput: string, prefix?: string): HTMLLabelElement {
        const newEl = document.createElement('label');
        if (type === TextFieldTypes.SINGLE) {
            newEl.className = "mdc-text-field mdc-text-field--outlined";
            newEl.id = id;
            newEl.innerHTML = `
            <span class="mdc-notched-outline">
              <span class="mdc-notched-outline__leading"></span>
              <span class="mdc-notched-outline__notch">
                <span class="mdc-floating-label" id="${id}-label">${label}</span>
              </span>
              <span class="mdc-notched-outline__trailing"></span>
            </span>
            `;
            if (prefix) {
                newEl.innerHTML += `
                <span class="mdc-text-field__affix mdc-text-field__affix--prefix">${prefix}</span>
                `;
            }
            newEl.innerHTML += `<input type="${typeInput}" class="mdc-text-field__input" aria-labelledby="${id}-label">`;
        } else if (type === TextFieldTypes.TEXTAREA) {
            newEl.className = "mdc-text-field mdc-text-field--outlined mdc-text-field--textarea mdc-text-field--no-label";
            newEl.id = id;
            newEl.innerHTML = `
            <span class="mdc-notched-outline">
              <span class="mdc-notched-outline__leading"></span>
              <span class="mdc-notched-outline__notch">
                <span class="mdc-floating-label" id="${id}-label">${label}</span>
              </span>
              <span class="mdc-notched-outline__trailing"></span>
            </span>
            <span class="mdc-text-field__resizer">
              <textarea class="mdc-text-field__input" rows="1" cols="40" aria-labelledby="${id}-label"></textarea>
            </span>
            `;
        }

        return newEl;
    }

    public checkData(dialogEls: DialogEls): boolean {
        let isValid = true;

        if (!dialogEls.quantity.getDefaultFoundation().getValue().trim() || dialogEls.quantity.getDefaultFoundation().getValue().trim() === "" || dialogEls.quantity.getDefaultFoundation().getValue().trim() === undefined ||
            Number(dialogEls.quantity.getDefaultFoundation().getValue().trim()) < 0) {
            isValid = false;
            dialogEls.quantity.getDefaultFoundation().setValid(false);
        }

        return isValid
    }
}

interface DialogStruct {
    header: HTMLElement;
    body: HTMLElement;
    buttonOk: HTMLElement;
    buttonOkLabel: HTMLElement;
}

interface DialogEls {
    productName: MDCTextField,
    quantity: MDCTextField,
}

enum TextFieldTypes {
    TEXTAREA,
    SINGLE
}