import {AdminPages, TopBarButtons} from "@/js/admin/admin_pages";
import {Api} from "@/js/admin/Api";
import {
    CustomerModel, Feedback,
    FeedbackModel,
    IngredientsModel,
    Products,
    ProductsModel,
    Stock,
    StockModel
} from "@/js/declarations";
import {ApiHost} from "@/js/vars";
import {MDCDialog} from "@material/dialog";
import {MDCRipple} from "@material/ripple";
import {MDCTextField} from "@material/textfield";
import {MDCSelect} from '@material/select';
import moment from "moment/moment";

export class PageFeedback {
    private _appContainer: HTMLElement;
    private _context: AdminPages;
    private _api: Api;
    private _pageContainer?: HTMLElement;
    private _dialog?: MDCDialog;
    private _dialogStruct?: DialogStruct;
    private _dialogEls?: DialogEls;
    private feedbackId?: string;
    private _pendingAction: boolean;

    constructor(appContainer: HTMLElement, adminPages: AdminPages) {
        this._appContainer = appContainer;
        this._api = new Api();
        this._context = adminPages;
        this._pendingAction = false;
        this._context.setTitle("Feedback");
        this.buildPage();
    }

    public destroy(): void {
        this._dialog?.root.remove();
        this._dialogStruct = undefined;
        this._dialogEls = undefined;
    }

    private async buildPage() {
        this._appContainer.innerHTML = "";
        const htmlData = await fetch("/template_admin_feedback");
        const htmlDataBody = await htmlData.text();
        this._appContainer.innerHTML = htmlDataBody;

        const dialogHtmlData = await fetch("template_admin_feedback_dialog");
        const dialogHtmlDataBody = await dialogHtmlData.text();

        const dialogEl = document.createElement('div');
        dialogEl.className = "mdc-dialog mdc-dialog--fullscreen";
        dialogEl.style.zIndex = "1000";
        dialogEl.innerHTML = dialogHtmlDataBody;
        dialogEl.id = "dialog-feedback";

        const docBody = document.querySelector('body');

        if (docBody) {
            docBody.insertAdjacentElement('afterbegin', dialogEl);
        }

        this._dialog = new MDCDialog(document.getElementById('dialog-feedback') || new HTMLElement());

        this._dialog.listen('MDCDialog:closing', () => {
            if (!this._pendingAction) {
                this.destroy();
                this.buildPage();
            }
        });

        this._pendingAction = false;

        this._dialogStruct = {
            header: document.getElementById("txt-head-dialog") || new HTMLElement(),
            body: document.getElementById("feedback-dialog-body") || new HTMLElement(),
            buttonOk: document.getElementById("button-dialog-ok") || new HTMLElement(),
            buttonOkLabel: document.getElementById("btn-label-dialog") || new HTMLElement()
        };

        this._pageContainer = document.querySelector('.container-gerenciador-feedback') || new HTMLElement();

        await this.getProducts();
    }

    private async getProducts() {
        const feedbacks: FeedbackModel = await this._api.getFromApi("/feedback") as FeedbackModel;
        if (feedbacks === null || feedbacks.error || feedbacks.feedbackList.length === 0) {
            if (this._pageContainer) {
                this._pageContainer.innerHTML += `
                <div class="mdc-typography--subtitle1 txt-empty-max-width no-feedback">Não há feedbacks registrados</div>
                `
            }
        } else {
            let avg;
            if (feedbacks.feedbackList.length === 1) {
                avg = feedbacks.feedbackList[0].rating;
            } else {
                avg = feedbacks.feedbackList.reduce((acc, curr) => acc + curr.rating, 0) / feedbacks.feedbackList.length;
            }
            if (this._pageContainer) {
                this._pageContainer.innerHTML += `
                <div class="mdc-typography--subtitle1 txt-empty-max-width no-feedback">Avaliação média: ${avg}⭐</div>
                `
            }
            for (const feedback of feedbacks.feedbackList) {
                const customer: CustomerModel = await this._api.getFromApi("/cliente", feedback.clientId) as CustomerModel;
                if (this._pageContainer) {
                    this._pageContainer.innerHTML += `
                <div class="mdc-card mdc-card--outlined card-feedback" data-feedback-id="${feedback.id}" id="${feedback.id}">
                  <div class="mdc-card__primary-action">
                    <div class="mdc-card__ripple"></div>
                  </div>
                  <div class="card-content">
                      <div class="mdc-card-wrapper__text-section">
                        <div class="mdc-card__title">${customer.clientes[0].nome} - ${feedback.rating}⭐</div>
                        <div class="mdc-card__subhead">${feedback.texto}</div>
                      </div>
                      <div class="mdc-card__actions">
                        <div class="mdc-card__action-buttons card-produto-action_buttons">
                          <button class="mdc-button mdc-card__action mdc-card__action--button button-card-feedback__edit" data-feedback-id="${feedback.id}">
                            <div class="mdc-button__ripple"></div>
                            <span class="mdc-button__label">Editar</span>
                          </button>
                        </div>
                      </div>
                    </div>
                </div>
                `;
                }
            }

            const cards = document.getElementsByClassName('card-feedback');
            if (cards) {
                for (let i = 0; i < cards.length; i++) {
                    const lastCard = cards[i] as HTMLElement;
                    const lastCardButtonEdit = lastCard.querySelector('.button-card-feedback__edit') || new HTMLElement();

                    new MDCRipple(lastCardButtonEdit);
                    lastCardButtonEdit.addEventListener('click', (event) => this.editFeedback(event, lastCard.id));
                }
            }
        }
    }

    private editFeedback = async (event: Event, feedbackId: string): Promise<void> => {
        if (this._dialog && this._dialogStruct) {
            this.feedbackId = feedbackId;
            this._dialogStruct.body.innerHTML = "";
            this._dialog.open();
            try {
                this._dialogStruct.buttonOk.removeEventListener('click', this.submitFeedback.bind(this));
            } catch (e) {}
            this._dialogStruct.header.innerText = "Editar Feedback";
            this._dialogStruct.buttonOkLabel.innerText = "Salvar";
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-feedback-dialog-subject", "Assunto", TextFieldTypes.SINGLE, "text"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-feedback-dialog-comment", "Comentário", TextFieldTypes.TEXTAREA, "text"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-feedback-dialog-date", "Data", TextFieldTypes.SINGLE, "text"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newSelector("select-feedback-dialog-rating", "Avaliação", "Avaliação", [
                {name: "1 ⭐ - Péssimo", value: "1"},
                {name: "2 ⭐ - Ruim", value: "2"},
                {name: "3 ⭐ - Regular", value: "3"},
                {name: "4 ⭐ - Bom", value: "4"},
                {name: "5 ⭐ - Excelente", value: "5"}
            ]));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newSelector("select-feedback-dialog-visibility", "Visibilidade", "Visibilidade", [
                {name: "Publico", value: "1"},
                {name: "Privado", value: "0"},
            ]));
            this._dialogEls = {
                subject: new MDCTextField(document.getElementById("txt-feedback-dialog-subject") || new HTMLElement()),
                comment: new MDCTextField(document.getElementById("txt-feedback-dialog-comment") || new HTMLElement()),
                rating: new MDCSelect(document.getElementById("select-feedback-dialog-rating") || new HTMLElement()),
                visibility: new MDCSelect(document.getElementById("select-feedback-dialog-visibility") || new HTMLElement()),
                date: new MDCTextField(document.getElementById("txt-feedback-dialog-date") || new HTMLElement())
            }

            const feedback: FeedbackModel = await this._api.getFromApi("/feedback", feedbackId) as FeedbackModel;

            this._dialogEls.subject.getDefaultFoundation().setValue(feedback.feedbackList[0].name);
            this._dialogEls.comment.getDefaultFoundation().setValue(feedback.feedbackList[0].texto);
            this._dialogEls.rating.setValue(feedback.feedbackList[0].rating.toString());
            this._dialogEls.date.getDefaultFoundation().setValue(moment.unix(feedback.feedbackList[0].date).format('DD/MM/YYYY HH:mm:ss'))

            if (feedback.feedbackList[0].publico) {
                this._dialogEls.visibility.setValue("1");
            } else {
                this._dialogEls.visibility.setValue("0");
            }

            this._dialogEls.subject.getDefaultFoundation().setDisabled(true);
            this._dialogEls.comment.getDefaultFoundation().setDisabled(true);
            this._dialogEls.rating.getDefaultFoundation().setDisabled(true);
            this._dialogEls.date.getDefaultFoundation().setDisabled(true);

            this._dialogStruct.buttonOk.addEventListener('click', this.submitFeedback.bind(this));
        }
    }

    private submitFeedback = async (): Promise<void> => {
        if (this._dialogStruct) {
            this._dialogStruct.buttonOk.setAttribute("disabled", "");
            this._pendingAction = true;
        }
        if (this.feedbackId && this._dialogEls) {
            // @ts-ignore
            const feedbackData: Feedback = {
                publico: Boolean(Number(this._dialogEls.visibility.getDefaultFoundation().getValue())),
            }

            let res = await this._api.putToApi("/feedback", feedbackData, this.feedbackId);

            this._dialog?.close();
            if (res["error"]) {
                this._context.showSnackbar("Erro ao Editar Feedback");
                this._dialogStruct?.buttonOk.removeAttribute("disabled");
            } else {
                this._context.showSnackbar("Feedback Editado com Sucesso");
                this.destroy();
                this.buildPage();
            }
        }
    }

    private newTextField(id: string, label: string, type: TextFieldTypes, typeInput: string, prefix?: string): HTMLLabelElement {
        const newEl = document.createElement('label');
        if (type === TextFieldTypes.SINGLE) {
            newEl.className = "mdc-text-field mdc-text-field--outlined";
            newEl.id = id;
            newEl.innerHTML = `
            <span class="mdc-notched-outline">
              <span class="mdc-notched-outline__leading"></span>
              <span class="mdc-notched-outline__notch">
                <span class="mdc-floating-label" id="${id}-label">${label}</span>
              </span>
              <span class="mdc-notched-outline__trailing"></span>
            </span>
            `;
            if (prefix) {
                newEl.innerHTML += `
                <span class="mdc-text-field__affix mdc-text-field__affix--prefix">${prefix}</span>
                `;
            }
            newEl.innerHTML += `<input type="${typeInput}" class="mdc-text-field__input" aria-labelledby="${id}-label">`;
        } else if (type === TextFieldTypes.TEXTAREA) {
            newEl.className = "mdc-text-field mdc-text-field--outlined mdc-text-field--textarea mdc-text-field--no-label";
            newEl.id = id;
            newEl.innerHTML = `
            <span class="mdc-notched-outline">
              <span class="mdc-notched-outline__leading"></span>
              <span class="mdc-notched-outline__notch">
                <span class="mdc-floating-label" id="${id}-label">${label}</span>
              </span>
              <span class="mdc-notched-outline__trailing"></span>
            </span>
            <span class="mdc-text-field__resizer">
              <textarea class="mdc-text-field__input" rows="1" cols="40" aria-labelledby="${id}-label"></textarea>
            </span>
            `;
        }

        return newEl;
    }

    private newSelector(id: string, label: string, selectLabel: string, items: SelectorItems[]): HTMLDivElement {
        let listItemsHtml = ""

        for (let i=0;i<items.length;i++) {
            listItemsHtml += `
            <li class="mdc-list-item" aria-selected="false" data-value="${items[i].value}" role="option">
              <span class="mdc-list-item__ripple"></span>
              <span class="mdc-list-item__text" style="align-content: center;">
                ${items[i].name}
              </span>
            </li>
            `;
        }

        const newEl = document.createElement('div');
        newEl.id = id;
        newEl.className = "mdc-select mdc-select--outlined";
        newEl.innerHTML = `
        <div class="mdc-select__anchor" aria-labelledby="${id}-select-label">
          <span class="mdc-notched-outline">
            <span class="mdc-notched-outline__leading"></span>
            <span class="mdc-notched-outline__notch">
              <span id="${id}-select-label" class="mdc-floating-label">${selectLabel}</span>
            </span>
            <span class="mdc-notched-outline__trailing"></span>
          </span>
          <span class="mdc-select__selected-text-container">
            <span id="demo-selected-text" class="mdc-select__selected-text"></span>
          </span>
          <span class="mdc-select__dropdown-icon">
            <svg
                class="mdc-select__dropdown-icon-graphic"
                viewBox="7 10 10 5" focusable="false">
              <polygon
                  class="mdc-select__dropdown-icon-inactive"
                  stroke="none"
                  fill-rule="evenodd"
                  points="7 10 12 15 17 10">
              </polygon>
              <polygon
                  class="mdc-select__dropdown-icon-active"
                  stroke="none"
                  fill-rule="evenodd"
                  points="7 15 12 10 17 15">
              </polygon>
            </svg>
          </span>
        </div>
        
        <div class="mdc-select__menu mdc-menu mdc-menu-surface mdc-menu-surface--fullwidth" style="z-index: 10000;">
        <ul class="mdc-list" role="listbox" aria-label="${label}" id="${id}-list">${listItemsHtml}</ul></div>
        `;

        return newEl
    }
}

interface DialogStruct {
    header: HTMLElement;
    body: HTMLElement;
    buttonOk: HTMLElement;
    buttonOkLabel: HTMLElement;
}

interface DialogEls {
    subject: MDCTextField,
    comment: MDCTextField,
    date: MDCTextField,
    rating: MDCSelect,
    visibility: MDCSelect,
}

interface SelectorItems {
    name: string,
    value: string
}

enum TextFieldTypes {
    TEXTAREA,
    SINGLE
}