import {AdminPages, TopBarButtons} from "@/js/admin/admin_pages";
import {Api} from "@/js/admin/Api";
import {Ingredient, IngredientsModel, Products, ProductsModel} from "@/js/declarations";
import {ApiHost} from "@/js/vars";
import {MDCDialog} from "@material/dialog";
import {MDCRipple} from "@material/ripple";
import {MDCTextField} from "@material/textfield";
import {MDCSelect} from '@material/select';

export class PageIngredients {
    private _appContainer: HTMLElement;
    private _context: AdminPages;
    private _api: Api;
    private _pageContainer?: HTMLElement;
    private _dialog?: MDCDialog;
    private _dialogStruct?: DialogStruct;
    private _pendingAction: boolean;

    constructor(appContainer: HTMLElement, adminPages: AdminPages) {
        this._appContainer = appContainer;
        this._api = new Api();
        this._context = adminPages;
        this._pendingAction = false;
        this._context.setTitle("Ingredientes");
        this.buildPage();
    }

    public destroy(): void {
        this._dialog?.root.remove();
        document.getElementById("btn-add-ingrediente")?.remove();
    }

    private async buildPage() {
        const htmlData = await fetch("/template_ingredientes");
        const htmlDataBody = await htmlData.text();
        this._appContainer.innerHTML = htmlDataBody;

        const dialogHtmlData = await fetch("/template_ingredientes_dialog");
        const dialogHtmlDataBody = await dialogHtmlData.text();

        const dialogEl = document.createElement('div');
        dialogEl.className = "mdc-dialog mdc-dialog--fullscreen";
        dialogEl.style.zIndex = "1000";
        dialogEl.innerHTML = dialogHtmlDataBody;
        dialogEl.id = "dialog-ingredientes";

        const docBody = document.querySelector('body');

        if (docBody) {
            docBody.insertAdjacentElement('afterbegin', dialogEl);
        }

        this._dialog = new MDCDialog(document.getElementById('dialog-ingredientes') || new HTMLElement());

        this._dialog.listen('MDCDialog:closing', () => {
            if (!this._pendingAction) {
                this.destroy();
                this.buildPage();
            }
        });

        this._dialogStruct = {
            header: document.getElementById("txt-head-dialog") || new HTMLElement(),
            body: document.getElementById("ingredientes-dialog-body") || new HTMLElement(),
            buttonOk: document.getElementById("button-dialog-ok") || new HTMLElement(),
            buttonOkLabel: document.getElementById("btn-label-dialog") || new HTMLElement(),
            buttonCancel: document.getElementById("button-dialog-cancel") || new HTMLElement(),
            buttonCancelLabel: document.getElementById("btn-cancel-label-dialog") || new HTMLElement()
        };

        this._pageContainer = document.querySelector('.container-gerenciador-ingredientes') || new HTMLElement();

        const buttons: TopBarButtons[] = [
            {
                label: "Adicionar Ingrediente",
                id: "btn-add-ingrediente",
                icon: "add"
            }
        ]
        this._context.setButtons(buttons);
        const btnAdd = document.getElementById('btn-add-ingrediente') as HTMLElement;
        btnAdd.addEventListener('click', this.addBtnHandler.bind(this));
        await this.getIngredientes();
    }

    private async getIngredientes() {
        const ingredients: IngredientsModel = await this._api.getFromApi("/ingredients/listar") as IngredientsModel;
        if (ingredients.data.length === 0) {
            if (this._pageContainer) {
                const table = this._pageContainer.querySelector('#table-root') as HTMLElement;
                table.remove();
                this._pageContainer.innerHTML += `
                <div class="mdc-typography--subtitle1 txt-empty-max-width no-ingredient">Não há ingredientes cadastrados</div>
                `
            }
        } else {
            ingredients.data.forEach((ingredient) => {
                if (this._pageContainer) {
                    const table = this._pageContainer.querySelector('#table-ingredients') as HTMLElement;
                    table.innerHTML += `
                <tr class="mdc-data-table__row row-ingredient" data-id="${ingredient.id}" id="${ingredient.id}">
                  <th class="mdc-data-table__cell" scope="row">${ingredient.name}</th>
                  <td class="mdc-data-table__cell">R$${ingredient.cost.toFixed(2)}</td>
                  <td class="mdc-data-table__cell">${ingredient.quantity}</td>
                </tr>
                `
                }
            })

            const rows = document.getElementsByClassName('row-ingredient');
            if (rows) {
                for (let i=0; i<rows.length; i++) {
                    rows[i].addEventListener('click', (event) => this.editIngredient(event, rows[i].id));
                }
            }
        }
    }

    private editIngredient = async (event: Event, ingredientId: string): Promise<void> => {
        if (this._dialog && this._dialogStruct) {
            this._dialogStruct.body.innerHTML = "";
            this._dialog.open();
            this._dialogStruct.header.innerText = "Editar Ingrediente";
            this._dialogStruct.buttonOkLabel.innerText = "Salvar";
            this._dialogStruct.buttonCancelLabel.innerText = "Remover";
            this._dialogStruct.buttonCancel.style.display = "inline-flex";
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-ingredient-dialog-name", "Nome", TextFieldTypes.SINGLE, "text"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-ingredient-dialog-price", "Preço", TextFieldTypes.SINGLE, "number", "R$"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-ingredient-dialog-quantity", "Quantidade", TextFieldTypes.SINGLE, "number"));
            const dialogEls: DialogEls = {
                name: new MDCTextField(document.getElementById("txt-ingredient-dialog-name") || new HTMLElement()),
                price: new MDCTextField(document.getElementById("txt-ingredient-dialog-price") || new HTMLElement()),
                quantity: new MDCTextField(document.getElementById("txt-ingredient-dialog-quantity") || new HTMLElement()),
            }

            const ingredientData: any = await this._api.getFromApi("/ingredients/get", ingredientId || "") as IngredientsModel;

            const item = ingredientData["data"] as Ingredient;

            dialogEls.name.getDefaultFoundation().setValue(item.name);
            dialogEls.price.getDefaultFoundation().setValue(Number(item.cost).toFixed(2));
            dialogEls.quantity.getDefaultFoundation().setValue(item.quantity.toString());

            this._dialogStruct.buttonOk.addEventListener('click', async () => {
                this._pendingAction = true;
                if (this.checkData(dialogEls)) {
                    if (this._dialogStruct) {
                        this._dialogStruct.buttonOk.setAttribute("disabled", "");
                    }
                    if (ingredientId) {
                        const ingredientData: Ingredient = {
                            id: ingredientId,
                            name: dialogEls.name.getDefaultFoundation().getValue(),
                            cost: Number(dialogEls.price.getDefaultFoundation().getValue()),
                            quantity: Number(dialogEls.quantity.getDefaultFoundation().getValue())
                        }

                        const res = await this._api.putToApi("/ingredients/editar", ingredientData, ingredientId);
                        this._dialog?.close();
                        if (res["error"]) {
                            this._context.showSnackbar("Erro ao Editar Ingrediente");
                            if (this._dialogStruct) {
                                this._dialogStruct.buttonOk.removeAttribute("disabled");
                            }
                        } else {
                            this._context.showSnackbar("Ingrediente Editado com Sucesso");
                            this.destroy();
                            this.buildPage();
                        }
                    }
                } else {
                    this._context.showSnackbar("Verifique os dados");
                }
            })

            this._dialogStruct.buttonCancel.addEventListener('click', () => this.deleteIngredient(ingredientId))
        }
    }

    private addBtnHandler = () => {
        if (this._dialog && this._dialogStruct) {
            this._dialogStruct.body.innerHTML = "";
            this._dialog.open();
            this._dialogStruct.header.innerText = "Adicionar Ingrediente";
            this._dialogStruct.buttonCancel.style.display = "none";
            this._dialogStruct.buttonOkLabel.innerText = "Salvar";
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-ingredient-dialog-name", "Nome", TextFieldTypes.SINGLE, "text"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-ingredient-dialog-price", "Preço", TextFieldTypes.SINGLE, "number", "R$"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-ingredient-dialog-quantity", "Quantidade", TextFieldTypes.SINGLE, "number"));
            const dialogEls: DialogEls = {
                name: new MDCTextField(document.getElementById("txt-ingredient-dialog-name") || new HTMLElement()),
                price: new MDCTextField(document.getElementById("txt-ingredient-dialog-price") || new HTMLElement()),
                quantity: new MDCTextField(document.getElementById("txt-ingredient-dialog-quantity") || new HTMLElement()),
            }

            this._dialogStruct.buttonOk.addEventListener('click', async () => {
                this._pendingAction = true;
                if (this.checkData(dialogEls)) {
                    if (this._dialogStruct) {
                        this._dialogStruct.buttonOk.setAttribute("disabled", "");
                    }
                    const ingredientData: { cost: number; quantity: number; name: string } = {
                        name: dialogEls.name.getDefaultFoundation().getValue(),
                        cost: Number(dialogEls.price.getDefaultFoundation().getValue()),
                        quantity: Number(dialogEls.quantity.getDefaultFoundation().getValue())
                    }

                    const res = await this._api.postToApi("/ingredients/registrar", ingredientData);
                    this._dialog?.close();
                    if (res["error"]) {
                        this._context.showSnackbar("Erro ao Adicionar Ingrediente");
                        if (this._dialogStruct) {
                            this._dialogStruct.buttonOk.removeAttribute("disabled");
                        }
                    } else {
                        this._context.showSnackbar("Ingrediente Adicionado com Sucesso");
                        this.destroy();
                        this.buildPage();
                    }
                } else {
                    this._context.showSnackbar("Verifique os dados");
                }
            })
        }
    }

    private async deleteIngredient(ingredientId: string): Promise<void> {
        if (ingredientId) {
            const res = await this._api.deleteFromApi("/ingredients/deletar", ingredientId);
            if (res["error"]) {
                this._context.showSnackbar("Erro ao Remover Ingrediente");
            } else {
                this.destroy();
                this.buildPage();
                this._context.showSnackbar("Ingrediente removido com sucesso");
            }
        }
    }

    public checkData(dialogEls: DialogEls): boolean {
        let isValid = true;

        if (!dialogEls.name.getDefaultFoundation().getValue().trim() || dialogEls.name.getDefaultFoundation().getValue().trim() === "" || dialogEls.name.getDefaultFoundation().getValue().trim() === undefined) {
            isValid = false;
            dialogEls.name.getDefaultFoundation().setValid(false);
        }
        if (!dialogEls.quantity.getDefaultFoundation().getValue().trim() || dialogEls.quantity.getDefaultFoundation().getValue().trim() === "" || dialogEls.quantity.getDefaultFoundation().getValue().trim() === undefined) {
            isValid = false;
            dialogEls.quantity.getDefaultFoundation().setValid(false);
        }
        if (!dialogEls.price.getDefaultFoundation().getValue().trim() || dialogEls.price.getDefaultFoundation().getValue().trim() === "" || dialogEls.price.getDefaultFoundation().getValue().trim() === undefined) {
            isValid = false;
            dialogEls.price.getDefaultFoundation().setValid(false);
        }

        return isValid
    }

    private newTextField(id: string, label: string, type: TextFieldTypes, typeInput: string, prefix?: string): HTMLLabelElement {
        const newEl = document.createElement('label');
        if (type === TextFieldTypes.SINGLE) {
            newEl.className = "mdc-text-field mdc-text-field--outlined";
            newEl.id = id;
            newEl.innerHTML = `
            <span class="mdc-notched-outline">
              <span class="mdc-notched-outline__leading"></span>
              <span class="mdc-notched-outline__notch">
                <span class="mdc-floating-label" id="${id}-label">${label}</span>
              </span>
              <span class="mdc-notched-outline__trailing"></span>
            </span>
            `;
            if (prefix) {
                newEl.innerHTML += `
                <span class="mdc-text-field__affix mdc-text-field__affix--prefix">${prefix}</span>
                `;
            }
            newEl.innerHTML += `<input type="${typeInput}" class="mdc-text-field__input" aria-labelledby="${id}-label">`;
        } else if (type === TextFieldTypes.TEXTAREA) {
            newEl.className = "mdc-text-field mdc-text-field--outlined mdc-text-field--textarea mdc-text-field--no-label";
            newEl.id = id;
            newEl.innerHTML = `
            <span class="mdc-notched-outline">
              <span class="mdc-notched-outline__leading"></span>
              <span class="mdc-notched-outline__notch">
                <span class="mdc-floating-label" id="${id}-label">${label}</span>
              </span>
              <span class="mdc-notched-outline__trailing"></span>
            </span>
            <span class="mdc-text-field__resizer">
              <textarea class="mdc-text-field__input" rows="1" cols="40" aria-labelledby="${id}-label"></textarea>
            </span>
            `;
        }

        return newEl;
    }
}

interface DialogStruct {
    header: HTMLElement;
    body: HTMLElement;
    buttonOk: HTMLElement;
    buttonOkLabel: HTMLElement;
    buttonCancel: HTMLElement;
    buttonCancelLabel: HTMLElement;
}

interface DialogEls {
    name: MDCTextField,
    price: MDCTextField,
    quantity: MDCTextField,
}

interface FileInput {
    root: HTMLElement,
    button: HTMLElement,
    file: HTMLInputElement
}

interface SelectorItems {
    name: string,
    value: string
}

enum TextFieldTypes {
    TEXTAREA,
    SINGLE
}