import {MDCDrawer} from "@material/drawer";
import {MDCList} from "@material/list";
import {MDCTopAppBar} from "@material/top-app-bar";
import {MDCSnackbar} from "@material/snackbar";
import {PageDeliveryFee} from "@/js/admin/admin_taxa_entrega";
import {PageProducts} from "@/js/admin/admin_produtos";
import {MDCRipple} from "@material/ripple";
import {PageIngredients} from "@/js/admin/admin_ingredientes";
import {PageOrders} from "@/js/admin/admin_pedidos";
import {PageStock} from "@/js/admin/admin_estoque";
import {PageFeedback} from "@/js/admin/admin_feedback";
import {PageSettings} from "@/js/admin/page_settings";

export class AdminPages {
    private _currentPage: AdminPagesEnum;
    private _drawerList: NodeList;
    private _appContainer: HTMLElement;
    private _drawer: MDCDrawer;
    private _list: MDCList;
    private _topAppBar: MDCTopAppBar;
    private _lastPage?: PageDeliveryFee | PageProducts | PageIngredients | PageOrders | PageStock | PageFeedback | PageSettings;
    private _snackbar: MDCSnackbar;
    private _barTitle: HTMLElement;
    private _barButtons: HTMLElement;

    constructor() {
        this._appContainer = document.getElementById('main-content') || new HTMLElement();
        this._drawerList = document.querySelectorAll('.admin-list-drawer') || new NodeList();
        this._drawer = MDCDrawer.attachTo(document.querySelector('.mdc-drawer') || new HTMLElement());
        this._list = MDCList.attachTo(document.querySelector('.mdc-list') || new HTMLElement());
        this._topAppBar = MDCTopAppBar.attachTo(document.getElementById('app-bar') || new HTMLElement());
        this._barTitle = document.getElementById('app-bar-title') || new HTMLElement();
        this._barButtons = document.getElementById('bar-buttons') || new HTMLElement();

        this._snackbar = new MDCSnackbar(document.querySelector('.mdc-snackbar') || new HTMLElement());

        this._list.wrapFocus = true;
        this._list.listElements.map((listItemEl) => new MDCRipple(listItemEl));
        this._topAppBar.setScrollTarget(document.getElementById('main-content-mdc') || new HTMLElement());
        this._topAppBar.listen('MDCTopAppBar:nav', () => {
            this._drawer.open = !this._drawer.open;
        });

        const page = this.getCurrentPage();
        if (page){
            this._currentPage = page;
        } else {
            this._currentPage = AdminPagesEnum.PRODUCT;
        }

        this.setPage(this._currentPage);

        for (let i=0;i<this._drawerList.length;i++) {
            this._drawerList[i].addEventListener('click', this.listHandler);
        }
    }

    public destroy() {
        for (let i=0;i<this._drawerList.length;i++) {
            this._drawerList[i].removeEventListener('click', this.listHandler);
        }
    }

    public reload() {
        const page = this.getCurrentPage();
        this._lastPage?.destroy();
        if (page) {
            this._currentPage = page;
            this.setPage(this._currentPage);
        }
    }

    private setPage(page: AdminPagesEnum): void {
        this._currentPage = page;
        if (this._lastPage) {
            this._lastPage.destroy();
        }
        this._appContainer.innerHTML = "";
        this.setUrlPage(page);
        this.activateListItem(page);
        switch (page) {
            case AdminPagesEnum.PRODUCT: {
                this._lastPage = new PageProducts(this._appContainer, this);
                break;
            }
            case AdminPagesEnum.INGREDIENT: {
                this._lastPage = new PageIngredients(this._appContainer, this);
                break;
            }
            case AdminPagesEnum.ORDER: {
                this._lastPage = new PageOrders(this._appContainer, this);
                break;
            }
            case AdminPagesEnum.INVENTORY: {
                this._lastPage = new PageStock(this._appContainer, this);
                break;
            }
            case AdminPagesEnum.SHIPPING: {
                this._lastPage = new PageDeliveryFee(this._appContainer, this);
                break;
            }
            case AdminPagesEnum.FEEDBACK: {
                this._lastPage = new PageFeedback(this._appContainer, this);
                break;
            }
            case AdminPagesEnum.SETTINGS: {
                this._lastPage = new PageSettings(this._appContainer, this);
                break;
            }
        }
    }

    private getCurrentPage(): AdminPagesEnum | null {
        const param = new URLSearchParams(window.location.search).get('page');
        if (param) {
            return Number(param) as AdminPagesEnum
        } else {
            return null
        }
    }

    private setUrlPage(page: AdminPagesEnum): void {
        const url = new URL(window.location.href);
        const search_params = url.searchParams;

        search_params.set('page', String(page));

        url.search = search_params.toString();

        const new_url = url.toString();
        window.history.pushState(`page=${page}`, 'Painel de Administração', new_url);
    }

    private listHandler = (event: Event) => {
        const target = event.target as HTMLElement;
        target.classList.add('mdc-list-item--activated');
        this._drawer.open = !this._drawer.open;
        this.setPage(Number(target.getAttribute('data-page')));
    }

    private activateListItem(page: AdminPagesEnum) {
        this._list.listElements.forEach((item) => {
            if(item.getAttribute('data-page') === String(page)) {
                item.classList.add('mdc-list-item--activated');
            } else {
                item.classList.remove('mdc-list-item--activated');
            }
        });
    }

    public showSnackbar(body: string) {
        this._snackbar.open();
        this._snackbar.labelText = body;
    }

    public setTitle(title: string) {
        this._barTitle.innerText = title;
    }

    public setButtons(buttons: TopBarButtons[]) {
        this._barButtons.innerHTML = '';
        buttons.forEach((btn) => {
            const button = document.createElement('button');
            button.className = "material-icons-outlined mdc-top-app-bar__action-item mdc-icon-button";
            button.innerText = btn.icon;
            button.id = btn.id;
            button.setAttribute('aria-label', btn.label);
            this._barButtons.appendChild(button);
        })
    }
}

export enum AdminPagesEnum {
    PRODUCT,
    INGREDIENT,
    ORDER,
    INVENTORY,
    SHIPPING,
    FEEDBACK,
    SETTINGS
}

export interface TopBarButtons {
    label: string,
    id: string,
    icon: string
}