import {AdminPages, TopBarButtons} from "@/js/admin/admin_pages";
import {Api} from "@/js/admin/Api";
import {
    CustomerModel,
    Ingredient,
    IngredientsModel,
    Order,
    OrderModel,
    Products,
    ProductsModel
} from "@/js/declarations";
import {ApiHost} from "@/js/vars";
import {MDCDialog} from "@material/dialog";
import {MDCRipple} from "@material/ripple";
import {MDCTextField} from "@material/textfield";
import {MDCSelect} from '@material/select';

export class PageOrders {
    private _appContainer: HTMLElement;
    private _context: AdminPages;
    private _api: Api;
    private _pageContainer?: HTMLElement;
    private _dialog?: MDCDialog;
    private _dialogStruct?: DialogStruct;
    private _pendingAction: boolean;

    constructor(appContainer: HTMLElement, adminPages: AdminPages) {
        this._appContainer = appContainer;
        this._api = new Api();
        this._context = adminPages;
        this._pendingAction = false;
        this._context.setTitle("Pedidos");
        this.buildPage();
    }

    public destroy(): void {
        this._dialog?.root.remove();
    }

    private async buildPage() {
        const htmlData = await fetch("/template_pedidos");
        const htmlDataBody = await htmlData.text();
        this._appContainer.innerHTML = htmlDataBody;

        const dialogHtmlData = await fetch("/template_pedidos_dialog");
        const dialogHtmlDataBody = await dialogHtmlData.text();

        const dialogEl = document.createElement('div');
        dialogEl.className = "mdc-dialog mdc-dialog--fullscreen";
        dialogEl.style.zIndex = "1000";
        dialogEl.innerHTML = dialogHtmlDataBody;
        dialogEl.id = "dialog-pedidos";

        const docBody = document.querySelector('body');

        if (docBody) {
            docBody.insertAdjacentElement('afterbegin', dialogEl);
        }

        this._dialog = new MDCDialog(document.getElementById('dialog-pedidos') || new HTMLElement());

        this._dialog.listen('MDCDialog:closing', () => {
            if (!this._pendingAction) {
                this.destroy();
                this.buildPage();
            }
        });

        this._dialogStruct = {
            header: document.getElementById("txt-head-dialog") || new HTMLElement(),
            body: document.getElementById("pedidos-dialog-body") || new HTMLElement(),
            buttonOk: document.getElementById("button-dialog-ok") || new HTMLElement(),
            buttonOkLabel: document.getElementById("btn-label-dialog") || new HTMLElement(),
            buttonCancel: document.getElementById("button-dialog-cancel") || new HTMLElement(),
            buttonCancelLabel: document.getElementById("btn-cancel-label-dialog") || new HTMLElement()
        };

        this._pageContainer = document.querySelector('.container-gerenciador-pedidos') || new HTMLElement();

        await this.getPedidos();
    }

    private async getPedidos() {
        const orders: OrderModel = await this._api.getFromApi("/pedido/list") as OrderModel;
        if (orders === null || orders.pedidos.length === 0) {
            if (this._pageContainer) {
                const table = this._pageContainer.querySelector('#table-root') as HTMLElement;
                table.remove();
                this._pageContainer.innerHTML += `
                <div class="mdc-typography--subtitle1 txt-empty-max-width no-order">Nenhum pedido foi realizado</div>
                `
            }
        } else {
            for (const order of orders.pedidos) {
                if (this._pageContainer) {
                    const customer: CustomerModel = await this._api.getFromApi("/cliente", order.idCliente) as CustomerModel;
                    const table = this._pageContainer.querySelector('#table-pedidos') as HTMLElement;
                    table.innerHTML += `
                <tr class="mdc-data-table__row row-pedido" data-id="${order.pedidoId}" id="${order.pedidoId}">
                  <th class="mdc-data-table__cell" scope="row">${customer.clientes[0].nome}</th>
                  <td class="mdc-data-table__cell">R$${Number(this.getOrderTotal(order))}</td>
                  <td class="mdc-data-table__cell">${this.getOrderStatus(order)}</td>
                </tr>
                `
                }
            }
            const rows = document.getElementsByClassName('row-pedido');
            if (rows) {
                for (let i = 0; i < rows.length; i++) {
                    const row = rows[i];
                    row.addEventListener('click', (event) => this.editOrder(event, row.id));
                }
            }
        }
    }

    private editOrder = async (event: Event, orderId: string): Promise<void> => {
        if (this._dialog && this._dialogStruct) {
            this._dialogStruct.body.innerHTML = "";
            this._dialog.open();
            this._dialogStruct.header.innerText = "Editar Pedido";
            this._dialogStruct.buttonOkLabel.innerText = "Salvar";
            this._dialogStruct.buttonCancelLabel.innerText = "Remover";
            this._dialogStruct.buttonCancel.style.display = "inline-flex";
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-order-dialog-total", "Total do Pedido", TextFieldTypes.SINGLE, "number", "R$"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newSelector("select-order-dialog-state", "Status do Pedido", "Status do Pedido", [{name: "Aguardando Pagamento", value: "0"}, {name: "Em Andamento", value: "1"}, {name: "Concluído", value: "2"}, {name: "Cancelado", value: "3"}]));

            const orderData: OrderModel = await this._api.getFromApi("/pedido", orderId || "") as OrderModel;
            const order: Order = orderData.pedidos[0];
            const customer: CustomerModel = await this._api.getFromApi("/cliente", order.idCliente) as CustomerModel;

            const listCustomer: HTMLElement = document.createElement('ul') as HTMLElement;
            listCustomer.className = "mdc-list mdc-list--two-line admin-list-customer";
            listCustomer.innerHTML = `
            <li class="mdc-list-item mdc-list-item__auto">
              <span class="mdc-list-item__ripple"></span>
              <span class="mdc-list-item__text">
                <span class="mdc-list-item__primary-text">${customer.clientes[0].nome}</span>
                <span class="mdc-list-item__secondary-text">Nome do Cliente</span>
              </span>
            </li>
            <li class="mdc-list-item mdc-list-item__auto">
              <span class="mdc-list-item__ripple"></span>
              <span class="mdc-list-item__text">
                <span class="mdc-list-item__primary-text">${customer.clientes[0].endereco}</span>
                <span class="mdc-list-item__secondary-text">Logradouro</span>
              </span>
            </li>
            <li class="mdc-list-item mdc-list-item__auto">
              <span class="mdc-list-item__ripple"></span>
              <span class="mdc-list-item__text">
                <span class="mdc-list-item__primary-text">${customer.clientes[0].numeroCasa}</span>
                <span class="mdc-list-item__secondary-text">Número da Residência</span>
              </span>
            </li>
            <li class="mdc-list-item mdc-list-item__auto">
              <span class="mdc-list-item__ripple"></span>
              <span class="mdc-list-item__text">
                <span class="mdc-list-item__primary-text">${customer.clientes[0].referenciaCasa}</span>
                <span class="mdc-list-item__secondary-text">Referência</span>
              </span>
            </li>
            <li class="mdc-list-item mdc-list-item__auto">
              <span class="mdc-list-item__ripple"></span>
              <span class="mdc-list-item__text">
                <span class="mdc-list-item__primary-text">${customer.clientes[0].numeroTelefone}</span>
                <span class="mdc-list-item__secondary-text">Número de Telefone</span>
              </span>
            </li>
            <li class="mdc-list-item mdc-list-item__auto">
              <span class="mdc-list-item__ripple"></span>
              <span class="mdc-list-item__text">
                <span class="mdc-list-item__primary-text">${customer.clientes[0].email}</span>
                <span class="mdc-list-item__secondary-text">Email</span>
              </span>
            </li>
            <li class="mdc-list-item mdc-list-item__auto">
              <span class="mdc-list-item__ripple"></span>
              <span class="mdc-list-item__text">
                <span class="mdc-list-item__primary-text">R$${order.deliveryFee.toFixed(2)}</span>
                <span class="mdc-list-item__secondary-text">Taxa de Entrega</span>
              </span>
            </li>
            <li class="mdc-list-item mdc-list-item__auto li-order-id" data-order-id="${order.pedidoId}">
              <span class="mdc-list-item__ripple"></span>
              <span class="mdc-list-item__text">
                <span class="mdc-list-item__primary-text">${order.pedidoId}</span>
                <span class="mdc-list-item__secondary-text">ID do pedido</span>
              </span>
            </li>
            `

            const listCustomerCt = document.createElement('div');
            listCustomerCt.style.height = "fit-content";

            listCustomer.id = "list-customer-dialog";
            listCustomerCt.insertAdjacentElement('afterbegin', listCustomer);
            this._dialogStruct.body.insertAdjacentElement('beforeend', listCustomerCt);

            const liOrderId = document.querySelector('.li-order-id') as HTMLElement;

            if (liOrderId) {
                liOrderId.addEventListener('click', () => this.copyToClipboard(order.pedidoId, "ID do Pedido Copiado", "Erro ao copiar ID do Pedido"));
            }

            const tableProducts: HTMLElement = document.createElement('div') as HTMLElement;
            tableProducts.id = "table-products-dialog";
            tableProducts.className = "mdc-data-table";
            tableProducts.innerHTML = `
            <div class="mdc-data-table__table-container">
                <table class="mdc-data-table__table" aria-label="Itens do Pedido">
                    <thead>
                    <tr class="mdc-data-table__header-row">
                        <th class="mdc-data-table__header-cell" role="columnheader" scope="col">Produto</th>
                        <th class="mdc-data-table__header-cell mdc-data-table__header-cell--numeric" role="columnheader" scope="col">Preço</th>
                        <th class="mdc-data-table__header-cell mdc-data-table__header-cell--numeric" role="columnheader" scope="col">Quantidade</th>
                        <th class="mdc-data-table__header-cell mdc-data-table__header-cell--numeric" role="columnheader" scope="col">Total</th>
                    </tr>
                    </thead>
                    <tbody class="mdc-data-table__content"></tbody>
                </table>
            </div>
            `

            const tableProductsCt = document.createElement('div');
            tableProductsCt.style.height = "fit-content";
            tableProductsCt.insertAdjacentElement('afterbegin', tableProducts);

            this._dialogStruct.body.insertAdjacentElement('beforeend', tableProductsCt);

            const dialogEls: DialogEls = {
                customerData: document.getElementById("list-customer-dialog") || new HTMLElement(),
                totalOrder: new MDCTextField(document.getElementById("txt-order-dialog-total") || new HTMLElement()),
                status: new MDCSelect(document.getElementById("select-order-dialog-state") || new HTMLElement()),
                orderProducts: document.getElementById("table-products-dialog") || new HTMLElement()
            }

            dialogEls.totalOrder.getDefaultFoundation().setValue(this.getOrderTotal(order).toFixed(2));
            dialogEls.totalOrder.getDefaultFoundation().setDisabled(true);
            dialogEls.status.setValue(order.statusPedido.toString());
            const tableItems = dialogEls.orderProducts.querySelector('.mdc-data-table__content') as HTMLElement;

            order.geladinhos.forEach((item) => {
                tableItems.innerHTML += `
                <tr class="mdc-data-table__row">
                  <th class="mdc-data-table__cell" scope="row">${item.nomeProduto}</th>
                  <td class="mdc-data-table__cell mdc-data-table__cell--numeric">R$${item.price.toFixed(2)}</td>
                  <td class="mdc-data-table__cell mdc-data-table__cell--numeric">${item.quantidade}</td>
                  <td class="mdc-data-table__cell mdc-data-table__cell--numeric">R$${(item.price*item.quantidade).toFixed(2)}</td>
                </tr>
                `
            })

            this._dialogStruct.buttonOk.addEventListener('click', async () => {
                this._pendingAction = true;
                if (this._dialogStruct) {
                    this._dialogStruct.buttonOk.setAttribute("disabled", "");
                }
                if (orderId) {
                    const orderData: { statusPedido: number } = {
                        statusPedido: Number(dialogEls.status.getDefaultFoundation().getValue())
                    }

                    const res = await this._api.putToApi("/pedido", orderData, orderId);
                    this._dialog?.close();
                    if (res["error"]) {
                        this._context.showSnackbar("Erro ao Editar Pedido");
                        if (this._dialogStruct) {
                            this._dialogStruct.buttonOk.removeAttribute("disabled");
                        }
                    } else {
                        this._context.showSnackbar("Pedido Editado com Sucesso");
                        this.destroy();
                        this.buildPage();
                    }
                }
            })

            this._dialogStruct.buttonCancel.addEventListener('click', () => this.deleteOrder(orderId))
        }
    }

    private copyToClipboard(text: string, message: string, message_error: string): void {
        if (text) {
            navigator.clipboard.writeText(text.toString()).then(() => {
                this._context.showSnackbar(message);
            }).catch(err => {
                console.error(message_error);
            });
        }
    }

    private async deleteOrder(orderId: string): Promise<void> {
        if (orderId) {
            const res = await this._api.deleteFromApi("/pedido", orderId);
            if (res["error"]) {
                this._context.showSnackbar("Erro ao Remover Pedido");
            } else {
                this.destroy();
                this.buildPage();
                this._context.showSnackbar("Pedido removido com sucesso");
            }
        }
    }

    private newSelector(id: string, label: string, selectLabel: string, items: SelectorItems[]): HTMLDivElement {
        let listItemsHtml = ""

        for (let i=0;i<items.length;i++) {
            listItemsHtml += `
            <li class="mdc-list-item" aria-selected="false" data-value="${items[i].value}" role="option">
              <span class="mdc-list-item__ripple"></span>
              <span class="mdc-list-item__text" style="align-content: center;">
                ${items[i].name}
              </span>
            </li>
            `;
        }

        const newEl = document.createElement('div');
        newEl.id = id;
        newEl.className = "mdc-select mdc-select--outlined";
        newEl.innerHTML = `
        <div class="mdc-select__anchor" aria-labelledby="${id}-select-label">
          <span class="mdc-notched-outline">
            <span class="mdc-notched-outline__leading"></span>
            <span class="mdc-notched-outline__notch">
              <span id="${id}-select-label" class="mdc-floating-label">${selectLabel}</span>
            </span>
            <span class="mdc-notched-outline__trailing"></span>
          </span>
          <span class="mdc-select__selected-text-container">
            <span id="demo-selected-text" class="mdc-select__selected-text"></span>
          </span>
          <span class="mdc-select__dropdown-icon">
            <svg
                class="mdc-select__dropdown-icon-graphic"
                viewBox="7 10 10 5" focusable="false">
              <polygon
                  class="mdc-select__dropdown-icon-inactive"
                  stroke="none"
                  fill-rule="evenodd"
                  points="7 10 12 15 17 10">
              </polygon>
              <polygon
                  class="mdc-select__dropdown-icon-active"
                  stroke="none"
                  fill-rule="evenodd"
                  points="7 15 12 10 17 15">
              </polygon>
            </svg>
          </span>
        </div>
        
        <div class="mdc-select__menu mdc-menu mdc-menu-surface mdc-menu-surface--fullwidth" style="z-index: 10000;">
        <ul class="mdc-list" role="listbox" aria-label="${label}" id="${id}-list">${listItemsHtml}</ul></div>
        `;

        return newEl
    }

    private newTextField(id: string, label: string, type: TextFieldTypes, typeInput: string, prefix?: string): HTMLLabelElement {
        const newEl = document.createElement('label');
        if (type === TextFieldTypes.SINGLE) {
            newEl.className = "mdc-text-field mdc-text-field--outlined";
            newEl.id = id;
            newEl.innerHTML = `
            <span class="mdc-notched-outline">
              <span class="mdc-notched-outline__leading"></span>
              <span class="mdc-notched-outline__notch">
                <span class="mdc-floating-label" id="${id}-label">${label}</span>
              </span>
              <span class="mdc-notched-outline__trailing"></span>
            </span>
            `;
            if (prefix) {
                newEl.innerHTML += `
                <span class="mdc-text-field__affix mdc-text-field__affix--prefix">${prefix}</span>
                `;
            }
            newEl.innerHTML += `<input type="${typeInput}" class="mdc-text-field__input" aria-labelledby="${id}-label">`;
        } else if (type === TextFieldTypes.TEXTAREA) {
            newEl.className = "mdc-text-field mdc-text-field--outlined mdc-text-field--textarea mdc-text-field--no-label";
            newEl.id = id;
            newEl.innerHTML = `
            <span class="mdc-notched-outline">
              <span class="mdc-notched-outline__leading"></span>
              <span class="mdc-notched-outline__notch">
                <span class="mdc-floating-label" id="${id}-label">${label}</span>
              </span>
              <span class="mdc-notched-outline__trailing"></span>
            </span>
            <span class="mdc-text-field__resizer">
              <textarea class="mdc-text-field__input" rows="1" cols="40" aria-labelledby="${id}-label"></textarea>
            </span>
            `;
        }

        return newEl;
    }

    private getOrderTotal(order: Order): number {
        let totalOrder = Number(0);

        order.geladinhos.forEach(item => {
            totalOrder += item.price * item.quantidade;
        });

        totalOrder += order.deliveryFee;

        return totalOrder;
    }

    private getOrderStatus(order: Order): string {
        switch (order.statusPedido) {
            case 0: {return "Aguardando Pagamento";}
            case 1: {return "Em Andamento";}
            case 2: {return "Concluído";}
            case 3: {return "Cancelado";}
            default: {return "Erro no status";}
        }
    }
}

interface DialogStruct {
    header: HTMLElement;
    body: HTMLElement;
    buttonOk: HTMLElement;
    buttonOkLabel: HTMLElement;
    buttonCancel: HTMLElement;
    buttonCancelLabel: HTMLElement;
}

interface DialogEls {
    customerData: HTMLElement,
    orderProducts: HTMLElement,
    totalOrder: MDCTextField,
    status: MDCSelect,
}

interface FileInput {
    root: HTMLElement,
    button: HTMLElement,
    file: HTMLInputElement
}

interface SelectorItems {
    name: string,
    value: string
}

enum TextFieldTypes {
    TEXTAREA,
    SINGLE
}