import { ApiHost } from "./vars";
import {MDCSnackbar} from '@material/snackbar';

const snackbar = new MDCSnackbar(document.querySelector('.mdc-snackbar'));
const fullscreenDiv = document.getElementById('fullscreen-div');
const modal = document.getElementById('modal');
const addBtn = document.getElementById('add-btn');
const listBtn = document.getElementById('list-btn');
const closeBtn = document.getElementById('close-btn');
const modalBtns = document.getElementById('modal-btns');
const form = document.getElementById('modal');
const ingredientList = document.querySelector('#ingredient-list tbody');


class Ingredient {
    constructor(id, name, cost, quantity) {
        this.id = id;
        this.name = name;
        this.cost = cost;
        this.quantity = quantity;
    }

    static async getIngredients() {
        const res = await fetch(`${ApiHost}/ingredients/listar`);
        const resData = await res.json();
        return resData["data"];
    }

    static async getIngredient(id) {
        const res = await fetch(`${ApiHost}/ingredients/get/${id}`);
        const resData = await res.json();
        return resData["data"];
    }

    static async addIngredient(ingredient) {
        try {
            const req = await fetch(`${ApiHost}/ingredients/registrar`, {
                method: "POST",
                headers: {
                    "Content-Type": "application/json"
                },
                body: JSON.stringify({
                    name: ingredient.name,
                    quantity: parseFloat(ingredient.quantity),
                    cost: parseFloat(ingredient.cost)
                })
            });
            const reqData = await req.json();
            //console.log(reqData);
            snackbar.open();
            snackbar.labelText = "Ingrediente adicionado com sucesso"
            return;
        } catch (error) {
            snackbar.open();
            snackbar.labelText = "Erro ao adicionar ingrediente"
        }
    }

    static async updateIngredient(ingredient) {
        try {
            const req = await fetch(`${ApiHost}/ingredients/editar/${ingredient.id}`, {
                method: "PUT",
                headers: {
                    "Content-Type": "application/json"
                },
                body: JSON.stringify(ingredient)
            });
            const reqData = await req.json();
            //console.log(reqData);
            snackbar.open();
            snackbar.labelText = "Ingrediente atualizado com sucesso"
            return;
        } catch (error) {
            snackbar.open();
            snackbar.labelText = "Erro ao atualizar ingrediente"
        }
    }

    static async deleteIngredient(id) {
        try {
            const req = await fetch(`${ApiHost}/ingredients/deletar/${id}`, {
                method: "DELETE"
            });
            const reqData = await req.json();
            snackbar.open();
            snackbar.labelText = "Ingrediente apagado com sucesso"
            //console.log(reqData);
            return;
        } catch (error) {
            snackbar.open();
            snackbar.labelText = "Erro ao apagar ingrediente"
        }
    }
}

class UI {
    static async showIngredientList() {
        const ingredients = await Ingredient.getIngredients();
        if(ingredients.length === 0) {
            ingredientList.innerHTML = `<tr><td colspan="4">Não há ingredientes cadastrados.</td></tr>`
        } else {
            ingredientList.innerHTML = "";
            ingredients.forEach(ingredient => UI.addToIngredientList(ingredient));
        }
    }

    static addToIngredientList(ingredient) {
        const tableRow = document.createElement('tr');
        tableRow.setAttribute('data-id', ingredient.id);
        tableRow.innerHTML = `
            <td>${ingredient.id}</td>
            <td>${ingredient.name}</td>
            <td>${ingredient.cost}</td>
            <td>${ingredient.quantity}</td>
        `;
        ingredientList.appendChild(tableRow);
    }

    static showModalData(id) {
        console.log("Mostrar dados do ingrediente com ID:", id);
    }

    static showModal() {
        modal.style.display = "block";
        fullscreenDiv.style.display = "block";
    }

    static closeModal() {
        modal.style.display = "none";
        fullscreenDiv.style.display = "none";
    }
}

window.addEventListener('DOMContentLoaded', () => {
    eventListeners();
    UI.showIngredientList();
});

function eventListeners() {
    addBtn.addEventListener('click', () => {
        form.reset();
        document.getElementById('modal-title').innerHTML = "Adicionar Ingrediente";
        UI.showModal();
        document.getElementById('modal-btns').innerHTML = `
            <button type="submit" id="save-btn">Salvar</button>
        `;
    });

    closeBtn.addEventListener('click', UI.closeModal);

    modalBtns.addEventListener('click', async (event) => {
        event.preventDefault();
        if (event.target.id == "save-btn") {
            let isFormValid = getFormData();
            if (!isFormValid) {
                snackbar.open();
                snackbar.labelText = "Certifique-se de que todos os dados são válidos"
            } else {
                let allIngredients = Ingredient.getIngredients();
                let lastIngredientId = allIngredients.length > 0 ? allIngredients[allIngredients.length - 1].id : 0;
                lastIngredientId++;

                const ingredientItem = new Ingredient(lastIngredientId, form.ingredient.value, form.cost_price.value, form.quantity.value);
                await Ingredient.addIngredient(ingredientItem);
                UI.closeModal();
                form.reset();
                await UI.showIngredientList();
            }
        } else if (event.target.id == "update-btn") {
            let isFormValid = getFormData();
            if (!isFormValid) {
                snackbar.open();
                snackbar.labelText = "Certifique-se de que todos os dados são válidos"
            } else {
                const editId = event.target.getAttribute('data-id');
                const updatedIngredient = new Ingredient(editId, form.ingredient.value, form.cost_price.value, form.quantity.value);
                await Ingredient.updateIngredient(updatedIngredient);
                UI.closeModal();
                await UI.showIngredientList();
            }
        }
    });

    ingredientList.addEventListener('click', async (event) => {
        if (event.target.parentElement.tagName === "TR") {
            const trElement = event.target.parentElement;
            const editID = trElement.dataset.id;
            const ingredientRow = document.querySelector(`tr[data-id="${editID}"]`);
            if (ingredientRow) {
                let ingrediente = await Ingredient.getIngredient(editID);
                form.reset();
                document.getElementById('modal-title').innerHTML = "Editar Ingrediente";
                UI.showModal();
                document.getElementById('modal-btns').innerHTML = `
                    <button type="submit" id="update-btn" data-id="${editID}">Atualizar</button>
                    <button type="button" id="delete-btn" data-id="${editID}">Excluir</button>
                `;
                
                form.ingredient.value = ingrediente.name;
                form.cost_price.value = ingrediente.cost;
                form.quantity.value = ingrediente.quantity;
            }
        }
    });

    modalBtns.addEventListener('click', async (event) => {
        if (event.target.id == 'delete-btn') {
            const ingredientId = event.target.dataset.id;
            await Ingredient.deleteIngredient(ingredientId);
            UI.closeModal();
            await UI.showIngredientList();
        }
    });
}

function getFormData() {
    let inputValidStatus = [
        form.ingredient.value,
        form.cost_price.value,
        form.quantity.value
    ];

    console.log(inputValidStatus);

    return inputValidStatus.includes("") ? false : true;
}
