import {ApiHost} from "@/js/vars";
import moment from "moment";
import {MDCSnackbar} from '@material/snackbar';

const snackbar = new MDCSnackbar(document.querySelector('.mdc-snackbar'));

class UI {
    static async fetchAndShowOrders() {
        const response = await fetch(`${ApiHost}/pedido/list`);
        const orderList = document.querySelector('#order-list tbody');
        
        try {
            const orders = await response.json();
            orderList.innerHTML = ''; // Limpar o conteúdo anterior
            orders["pedidos"].forEach(order => UI.addToOrderList(order));
        } catch (error) {
            orderList.innerHTML = `<tr><td colspan="4">Nenhum pedido encontrado</td></tr>`;
        }
    }

    static async addToOrderList(order) {
        const orderList = document.querySelector('#order-list tbody');
        const tableRow = document.createElement('tr');
        tableRow.setAttribute('data-id', order.pedidoId);

        let statusPedido = "";

        switch (order.statusPedido) {
            case 0: {statusPedido = "Aguardando Pagamento"; break;}
            case 1: {statusPedido = "Em Andamento"; break;}
            case 2: {statusPedido = "Concluído"; break;}
            case 3: {statusPedido = "Cancelado"; break;}
            default: {statusPedido = "Erro no status"; break;}
        }

        const customer = await fetch(`${ApiHost}/cliente/${order.idCliente}`);
        const customerData = await customer.json();

        let totalOrder = this.getTotalPrice(order);

        //const orders = order.geladinhos.map(item => `${item.titulo} (${item.quantidade})`);
        tableRow.innerHTML = `
            <td>${order.pedidoId}</td>
            <td>${customerData["clientes"][0]["nome"]}</td>
            <td>${statusPedido}</td>
            <td>R$${totalOrder}</td>
        `;
        orderList.appendChild(tableRow);
    }

    static async showModal(id) {
        const order = await fetch(`${ApiHost}/pedido/${id}`);
        let orderData = await order.json();
        orderData = orderData["pedidos"][0];
        const customer = await fetch(`${ApiHost}/cliente/${orderData.idCliente}`);
        let customerData = await customer.json();
        customerData = customerData["clientes"][0];

        const modal = document.getElementById('modal');
        modal.querySelector("#id").value = orderData.pedidoId;
        modal.querySelector("#nomeCliente").value = customerData.nome;
        modal.querySelector("#precoTotal").value = this.getTotalPrice(orderData);
        modal.querySelector("#endereco").value = customerData.endereco;
        modal.querySelector("#dataHora").value = this.parseDate(orderData.date);
        modal.querySelector("#statusPedido").value = orderData.statusPedido;

        this.buildModalOrderList(orderData);

        const fullscreenDiv = document.getElementById('fullscreen-div');
        modal.style.display = "block";
        fullscreenDiv.style.display = "block";
    }

    static buildModalOrderList(order) { 
        const orderList = document.querySelector('#table-items-modal');
        orderList.innerHTML = '';
        order["geladinhos"].forEach(item => { 
            const tableRow = document.createElement('tr');
            tableRow.setAttribute('data-id', item.idProduto);
            tableRow.innerHTML = `
                <td>${item.nomeProduto}</td>
                <td>${item.quantidade}</td>
                <td>R$${item.price}</td>
                <td>R$${item.price * item.quantidade}</td>
            `;
            orderList.appendChild(tableRow);
        });
    }

    static closeModal() {
        const modal = document.getElementById('modal');
        const fullscreenDiv = document.getElementById('fullscreen-div');
        modal.style.display = "none";
        fullscreenDiv.style.display = "none";
    }

    static getTotalPrice(order) {
        let totalOrder = new Number(0);

        order["geladinhos"].forEach(item => {
            totalOrder += item.price * item.quantidade;
        });

        return totalOrder;
    }

    static parseDate(unixTimestamp) {
        var date = moment(unixTimestamp * 1000);
        return date.format("DD/MM/YYYY, h:mm:ss A");
    }
}

class Order {
    static async updateOrder(id, newStatus) {
        const response = await fetch(`${ApiHost}/pedido/${id}`, {
            method: 'PUT',
            headers: {'Content-Type': 'application/json'},
            body: JSON.stringify({ statusPedido: newStatus })
        });
        return response.ok;
    }

    static async deleteOrder(id) {
        const response = await fetch(`${ApiHost}/pedido/${id}`, {
            method: 'DELETE'
        });
        return response.ok;
    }
}

function eventListeners() {
    const closeBtn = document.getElementById('close-btn');
    const modalBtns = document.getElementById('modal-btns');
    const orderList = document.querySelector('#order-list tbody');

    closeBtn.addEventListener('click', UI.closeModal);

    modalBtns.addEventListener('click', async (event) => {
        event.preventDefault();
        const id = document.querySelector('#id').value;
        if (event.target.id === 'update-btn') {
            const newStatus = document.getElementById('statusPedido').value;
            const success = await Order.updateOrder(id, newStatus);
            if (success) {
                snackbar.open();
                snackbar.labelText = "Pedido atualizado com sucesso!";
                UI.fetchAndShowOrders();
                UI.closeModal();
            } else {
                snackbar.open();
                snackbar.labelText = "Erro ao atualizar pedido";
            }
        } else if (event.target.id === 'delete-btn') {
            const success = await Order.deleteOrder(id);
            if (success) {
                snackbar.open();
                snackbar.labelText = "Pedido apagado com sucesso!";
                UI.fetchAndShowOrders();
                UI.closeModal();
            } else {
                snackbar.open();
                snackbar.labelText = "Erro ao apagar pedido";
            }
        }
    });

    orderList.addEventListener('click', (event) => {
        let trElement = event.target.closest('tr');
        const viewID = trElement.getAttribute('data-id');
        UI.showModal(viewID);
    });
}

window.addEventListener('DOMContentLoaded', () => {
    eventListeners();
    UI.fetchAndShowOrders();
});
