import {
    CustomerModel, Feedback,
    OrderModel,
    OrderProductsModel,
    Products,
    ProductsModel,
    Stock,
    StockModel
} from "@/js/declarations";
import {MDCDialog} from "@material/dialog";
import {MDCDataTable} from '@material/data-table';
import {ApiHost, MainDocumentUrl} from "@/js/vars";
import moment from "moment";
import {MDCRipple} from "@material/ripple";
import {MDCTextField} from "@material/textfield";
import {MDCSelect} from "@material/select";
import {MDCSnackbar} from "@material/snackbar";

class OrderConfirmation {
    private _orderId: string | null;
    private _orderData?: OrderModel;
    private _customerData?: CustomerModel;
    private _orderDetailsElements?: OrderDetailsElements;
    private _container: HTMLElement;
    private _alertDialog: MDCDialog;
    private _itemsTable?: MDCDataTable;
    private _btnCancel?: HTMLElement;
    private _btnFeedback?: HTMLElement;
    private _dialog?: MDCDialog;
    private _dialogStruct?: DialogStruct;
    private _dialogEls?: DialogEls;
    private _imgPix?: HTMLImageElement;
    private _btnPix?: HTMLElement;
    private _pixCopiaECola?: String;
    private _snackbar?: MDCSnackbar;

    constructor() {
        this._orderId = this.getOrderId();
        this._container = document.querySelector('.container') || new HTMLElement();
        this._alertDialog = new MDCDialog(document.getElementById('order-alert-dialog') || new HTMLElement());

        if (this._orderId) {
            this.buildPage();
        } else {
            this.noOrderFound();
        }
    }

    private async buildPage(){
        const res = await fetch(`/template_order_confirmation`);
        const resHtmlData = await res.text();

        this._snackbar = new MDCSnackbar(document.querySelector('.mdc-snackbar') || new HTMLElement());

        const dialogHtmlData = await fetch("template_feedback_dialog");
        const dialogHtmlDataBody = await dialogHtmlData.text();

        const dialogEl = document.createElement('div');
        dialogEl.className = "mdc-dialog mdc-dialog--fullscreen";
        dialogEl.style.zIndex = "1000";
        dialogEl.innerHTML = dialogHtmlDataBody;
        dialogEl.id = "dialog-feedback";

        const docBody = document.querySelector('body');

        if (docBody) {
            docBody.insertAdjacentElement('afterbegin', dialogEl);
        }

        this._dialog = new MDCDialog(document.getElementById('dialog-feedback') || new HTMLElement());

        this._dialogStruct = {
            header: document.getElementById("txt-head-dialog") || new HTMLElement(),
            body: document.getElementById("feedback-dialog-body") || new HTMLElement(),
            buttonOk: document.getElementById("button-dialog-ok") || new HTMLElement(),
            buttonOkLabel: document.getElementById("btn-label-dialog") || new HTMLElement()
        };

        this._container.innerHTML = resHtmlData;

        this._orderDetailsElements = {
            txtOrderId: this._container.querySelector('#txt-order-id') || new HTMLElement(),
            txtOrderAddress: this._container.querySelector('#txt-order-address') || new HTMLElement(),
            txtOrderCustomer: this._container.querySelector('#txt-order-customer') || new HTMLElement(),
            txtOrderDate: this._container.querySelector('#txt-order-date') || new HTMLElement(),
            txtOrderStatus: this._container.querySelector('#txt-order-status') || new HTMLElement(),
            txtOrderDelivery: this._container.querySelector('#txt-order-delivery') || new HTMLElement(),
            txtOrderTotal: this._container.querySelector('#txt-order-total') || new HTMLElement(),
        };

        this._itemsTable = new MDCDataTable(document.querySelector('.mdc-data-table') || new HTMLElement());
        this._btnCancel = document.querySelector('#btn-order-cancel') || new HTMLElement();
        this._btnFeedback = document.querySelector('#btn-add-feedback') || new HTMLElement();
        this._imgPix = document.querySelector('#img-pix') as HTMLImageElement|| new HTMLImageElement();
        this._btnPix = document.querySelector('#btn-order-pix') || new HTMLElement();

        this._imgPix.src = `${ApiHost}/pedido/${this._orderId}/pix/qr`;

        new MDCRipple(this._btnCancel);
        new MDCRipple(this._btnFeedback);

        try {
            const orderData = await fetch(`${ApiHost}/pedido/${this._orderId}`);
            this._orderData = await orderData.json() as OrderModel;
            const customerData = await fetch(`${ApiHost}/cliente/${this._orderData.pedidos[0].idCliente}`);
            this._customerData = await customerData.json() as CustomerModel;
            await this.addOrderDetails();
        } catch (e) {
            this.noOrderFound();
        }
        this._dialogStruct.buttonOk.addEventListener('click', this.submitFeedback.bind(this));
        this._pixCopiaECola = await fetch(`${ApiHost}/pedido/${this._orderId}/pix/code`).then(res => res.text());
        this._btnPix.addEventListener('click', this.copyPixCode.bind(this));
    }

    private copyPixCode(): void {
        if (this._pixCopiaECola) {
            navigator.clipboard.writeText(this._pixCopiaECola.toString()).then(() => {
                this.showSnackbar("Código PIX copiado");
            }).catch(err => {
                console.error('Erro ao copiar o código PIX: ', err);
            });
        }
    }

    public showSnackbar(body: string) {
        if (this._snackbar) {
            this._snackbar.open();
            this._snackbar.labelText = body;
        }
    }

    private async addOrderDetails() {
        if (this._orderData && this._orderDetailsElements && this._customerData) {
            this._orderDetailsElements.txtOrderId.innerText = this._orderData.pedidos[0].pedidoId;
            this._orderDetailsElements.txtOrderDate.innerText = moment.unix(this._orderData.pedidos[0].date).format('DD/MM/YYYY HH:mm:ss');
            this._orderDetailsElements.txtOrderCustomer.innerText = this._customerData.clientes[0].nome;
            this._orderDetailsElements.txtOrderAddress.innerText = this._customerData.clientes[0].endereco;
            this._orderDetailsElements.txtOrderDelivery.innerText = `R$${this._orderData.pedidos[0].deliveryFee.toFixed(2)}`;
            this._orderDetailsElements.txtOrderTotal.innerText = `R$${this.calcTotal()?.toFixed(2) || 0}`

            switch (this._orderData.pedidos[0].statusPedido) {
                case 0: {
                    this._orderDetailsElements.txtOrderStatus.innerText = "Aguardando Pagamento";
                    break;
                }
                case 1: {
                    this._orderDetailsElements.txtOrderStatus.innerText = "Em Andamento";
                    break;
                }
                case 2: {
                    this._orderDetailsElements.txtOrderStatus.innerText = "Concluído";
                    break;
                }
                case 3: {
                    this._orderDetailsElements.txtOrderStatus.innerText = "Cancelado";
                    break;
                }
            }

            if (this._itemsTable){
                const content = this._itemsTable.root.querySelector('.mdc-data-table__content') || new Element;
                this._orderData.pedidos[0].geladinhos.forEach((item: OrderProductsModel) => {
                    content.innerHTML += `
                    <tr class="mdc-data-table__row">
                      <th class="mdc-data-table__cell" scope="row">${item.nomeProduto}</th>
                      <td class="mdc-data-table__cell mdc-data-table__cell--numeric">R$${item.price.toFixed(2)}</td>
                      <td class="mdc-data-table__cell mdc-data-table__cell--numeric">${item.quantidade}</td>
                      <td class="mdc-data-table__cell mdc-data-table__cell--numeric">R$${(item.price*item.quantidade).toFixed(2)}</td>
                    </tr>
                    `
                })
            }

            if (this._orderData.pedidos[0].statusPedido === 0 && this._btnCancel) {
                this._btnCancel.removeAttribute('disabled');
                this._btnCancel.addEventListener('click', this.cancelOrder)
            }
            if (this._btnFeedback) {
                this._btnFeedback.addEventListener('click', this.addFeedback.bind(this));
            }
        }
    }

    private cancelOrder = (): void => {

        const actions = document.querySelector('#order-alert-dialog-actions') || new Element();
        actions.innerHTML = `
        <button type="button" class="mdc-button mdc-dialog__button" data-mdc-dialog-action="cancel">
            <div class="mdc-button__ripple"></div>
            <span class="mdc-button__label">Voltar</span>
        </button>
        <button type="button" class="mdc-button mdc-dialog__button" id="btn-dialog-cancel">
            <div class="mdc-button__ripple"></div>
            <span class="mdc-button__label">Cancelar</span>
        </button>
        `

        this.openAlertDialog("Tem certeza que deseja cancelar esse pedido?");

        let btnCancel: HTMLElement;

        this._alertDialog.listen('MDCDialog:opened', () => {
            btnCancel = document.getElementById('btn-dialog-cancel') || new HTMLElement();
            const btns = document.querySelectorAll('.mdc-dialog__button');

            for (let i = 0; i < btns.length; i++) {
                new MDCRipple(btns[i]);
            }

            btnCancel.addEventListener('click', this.acceptCancel);
        })

        this._alertDialog.listen('MDCDialog:closing', () => {
            btnCancel.removeEventListener('click', this.acceptCancel);
        })
    }

    private acceptCancel =  async () => {
        const response = await fetch(`${ApiHost}/pedido/${this._orderId}`, {
            method: 'PUT',
            headers: {'Content-Type': 'application/json'},
            body: JSON.stringify({ statusPedido: 3 })
        });

        window.location.reload();
    }

    private noOrderFound() {
        this._container.innerHTML = "";
        this.openAlertDialog("O pedido não foi encontrado, favor verificar os dados.");
        const btns = document.querySelectorAll('.mdc-dialog__button');

        for (let i = 0; i < btns.length; i++) {
            new MDCRipple(btns[i]);
        }
        this._alertDialog.listen('MDCDialog:closing', function (event) {
            window.location.href = MainDocumentUrl;
        })
    }

    private getOrderId(): string | null {
        const param = new URLSearchParams(window.location.search).get('order');
        if (param) {
            return param
        } else {
            return null
        }
    }

    private openAlertDialog(body: string): void {
        const dialogText = document.getElementById('order-alert-dialog-content');
        if (dialogText) {
            dialogText.innerText = body;
        }
        this._alertDialog.open();
    }

    private calcTotal(): number | null {
        let total: number = 0;
        if (this._orderData) {
            this._orderData.pedidos[0].geladinhos.forEach((item) => {
                total += item.price;
            })
            total += this._orderData.pedidos[0].deliveryFee;
            return total
        } else {
            return null
        }
    }

    private addFeedback = (): void => {
        if (this._dialog && this._dialogStruct) {
            this._dialogStruct.body.innerHTML = "";
            this._dialog.open();
            this._dialogStruct.header.innerText = "Avaliar";
            this._dialogStruct.buttonOkLabel.innerText = "Salvar";
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-feedback-dialog-subject", "Assunto", TextFieldTypes.SINGLE, "text"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-feedback-dialog-comment", "Comentário", TextFieldTypes.TEXTAREA, "text"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newSelector("select-feedback-dialog-rating", "Avaliação", "Avaliação", [
                {name: "1 ⭐ - Péssimo", value: "1"},
                {name: "2 ⭐ - Ruim", value: "2"},
                {name: "3 ⭐ - Regular", value: "3"},
                {name: "4 ⭐ - Bom", value: "4"},
                {name: "5 ⭐ - Excelente", value: "5"}
            ]));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newSelector("select-feedback-dialog-visibility", "Visibilidade", "Visibilidade", [
                {name: "Publico", value: "1"},
                {name: "Privado", value: "0"},
            ]));
            this._dialogEls = {
                subject: new MDCTextField(document.getElementById("txt-feedback-dialog-subject") || new HTMLElement()),
                comment: new MDCTextField(document.getElementById("txt-feedback-dialog-comment") || new HTMLElement()),
                rating: new MDCSelect(document.getElementById("select-feedback-dialog-rating") || new HTMLElement()),
                visibility: new MDCSelect(document.getElementById("select-feedback-dialog-visibility") || new HTMLElement())
            }
        }
    }

    private checkFeedback(): boolean {
        let valid = true;
        if (this._dialogEls) {
            if (!this._dialogEls.subject.getDefaultFoundation().getValue().trim()) {
                this._dialogEls.subject.getDefaultFoundation().setValid(false)
                valid = false;
            }
            if (!this._dialogEls.comment.getDefaultFoundation().getValue().trim()) {
                this._dialogEls.comment.getDefaultFoundation().setValid(false)
                valid = false;
            }
            if (!this._dialogEls.rating.getDefaultFoundation().getValue()) {
                this._dialogEls.rating.getDefaultFoundation().setValid(false)
                valid = false;
            } else {
                this._dialogEls.rating.getDefaultFoundation().setValid(true)
            }
            if (!this._dialogEls.visibility.getDefaultFoundation().getValue()) {
                this._dialogEls.visibility.getDefaultFoundation().setValid(false)
                valid = false;
            } else {
                this._dialogEls.visibility.getDefaultFoundation().setValid(true)
            }
        } else {
            valid = false;
        }
        return valid;
    }

    private submitFeedback = async () => {
        if (this.checkFeedback()) {
            if (this._dialogStruct) {
                this._dialogStruct.buttonOk.setAttribute("disabled", "");
            }
            if (this._customerData && this._customerData.clientes[0].idCliente && this._dialogEls) {
                // @ts-ignore
                const feedbackData: Feedback = {
                    clientId: this._customerData.clientes[0].idCliente,
                    name: this._dialogEls.subject.getDefaultFoundation().getValue(),
                    publico: Boolean(Number(this._dialogEls.visibility.getDefaultFoundation().getValue())),
                    rating: Number(this._dialogEls.rating.getDefaultFoundation().getValue()),
                    texto: this._dialogEls.comment.getDefaultFoundation().getValue()
                }

                let res = await fetch(`${ApiHost}/feedback`, {
                    method: 'POST',
                    headers: {'Content-Type': 'application/json'},
                    body: JSON.stringify(feedbackData)
                });

                let resData = await res.json();

                this._dialog?.close();
                if (!res.ok || resData["error"]) {
                    this.openAlertDialog("Erro ao enviar o feedback, tente novamente mais tarde.");
                    this._dialogStruct?.buttonOk.removeAttribute("disabled");
                } else {
                    this.openAlertDialog("Feedback enviado com sucesso!");
                    this._dialogStruct?.buttonOk.removeAttribute("disabled");
                }
            }
        }
    }

    private newTextField(id: string, label: string, type: TextFieldTypes, typeInput: string, prefix?: string): HTMLLabelElement {
        const newEl = document.createElement('label');
        if (type === TextFieldTypes.SINGLE) {
            newEl.className = "mdc-text-field mdc-text-field--outlined";
            newEl.id = id;
            newEl.innerHTML = `
            <span class="mdc-notched-outline">
              <span class="mdc-notched-outline__leading"></span>
              <span class="mdc-notched-outline__notch">
                <span class="mdc-floating-label" id="${id}-label">${label}</span>
              </span>
              <span class="mdc-notched-outline__trailing"></span>
            </span>
            `;
            if (prefix) {
                newEl.innerHTML += `
                <span class="mdc-text-field__affix mdc-text-field__affix--prefix">${prefix}</span>
                `;
            }
            newEl.innerHTML += `<input type="${typeInput}" class="mdc-text-field__input" aria-labelledby="${id}-label">`;
        } else if (type === TextFieldTypes.TEXTAREA) {
            newEl.className = "mdc-text-field mdc-text-field--outlined mdc-text-field--textarea mdc-text-field--no-label";
            newEl.id = id;
            newEl.innerHTML = `
            <span class="mdc-notched-outline">
              <span class="mdc-notched-outline__leading"></span>
              <span class="mdc-notched-outline__notch">
                <span class="mdc-floating-label" id="${id}-label">${label}</span>
              </span>
              <span class="mdc-notched-outline__trailing"></span>
            </span>
            <span class="mdc-text-field__resizer">
              <textarea class="mdc-text-field__input" rows="1" cols="40" aria-labelledby="${id}-label"></textarea>
            </span>
            `;
        }

        return newEl;
    }

    private newSelector(id: string, label: string, selectLabel: string, items: SelectorItems[]): HTMLDivElement {
        let listItemsHtml = ""

        for (let i=0;i<items.length;i++) {
            listItemsHtml += `
            <li class="mdc-list-item" aria-selected="false" data-value="${items[i].value}" role="option">
              <span class="mdc-list-item__ripple"></span>
              <span class="mdc-list-item__text" style="align-content: center;">
                ${items[i].name}
              </span>
            </li>
            `;
        }

        const newEl = document.createElement('div');
        newEl.id = id;
        newEl.className = "mdc-select mdc-select--outlined";
        newEl.innerHTML = `
        <div class="mdc-select__anchor" aria-labelledby="${id}-select-label">
          <span class="mdc-notched-outline">
            <span class="mdc-notched-outline__leading"></span>
            <span class="mdc-notched-outline__notch">
              <span id="${id}-select-label" class="mdc-floating-label">${selectLabel}</span>
            </span>
            <span class="mdc-notched-outline__trailing"></span>
          </span>
          <span class="mdc-select__selected-text-container">
            <span id="demo-selected-text" class="mdc-select__selected-text"></span>
          </span>
          <span class="mdc-select__dropdown-icon">
            <svg
                class="mdc-select__dropdown-icon-graphic"
                viewBox="7 10 10 5" focusable="false">
              <polygon
                  class="mdc-select__dropdown-icon-inactive"
                  stroke="none"
                  fill-rule="evenodd"
                  points="7 10 12 15 17 10">
              </polygon>
              <polygon
                  class="mdc-select__dropdown-icon-active"
                  stroke="none"
                  fill-rule="evenodd"
                  points="7 15 12 10 17 15">
              </polygon>
            </svg>
          </span>
        </div>
        
        <div class="mdc-select__menu mdc-menu mdc-menu-surface mdc-menu-surface--fullwidth" style="z-index: 10000;">
        <ul class="mdc-list" role="listbox" aria-label="${label}" id="${id}-list">${listItemsHtml}</ul></div>
        `;

        return newEl
    }
}

interface OrderDetailsElements {
    txtOrderId: HTMLElement;
    txtOrderDate: HTMLElement;
    txtOrderCustomer: HTMLElement;
    txtOrderAddress: HTMLElement;
    txtOrderStatus: HTMLElement;
    txtOrderDelivery: HTMLElement;
    txtOrderTotal: HTMLElement;
}

interface DialogStruct {
    header: HTMLElement;
    body: HTMLElement;
    buttonOk: HTMLElement;
    buttonOkLabel: HTMLElement;
}

interface DialogEls {
    subject: MDCTextField,
    comment: MDCTextField,
    rating: MDCSelect,
    visibility: MDCSelect,
}

interface SelectorItems {
    name: string,
    value: string
}

enum TextFieldTypes {
    TEXTAREA,
    SINGLE
}

new OrderConfirmation();